/*
	Stefan Gabriela
	333CA
*/
#ifndef __X0_H__
#define __X0_H__

#define F_CPU 16000000

#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include <avr/sleep.h>
#include <stdlib.h>
#include <stdint.h>
#include <stdio.h>

#define RED 0
#define GREEN 1
#define EMPTY 2

#define UP		0
#define RIGHT	1
#define DOWN	2
#define LEFT	3

#define MAX_MOVES	9

#define UP_BUTTON		PB4
#define LEFT_BUTTON		PB6
#define MOVE_BUTTON		PB5
#define RIGHT_BUTTON	PB7
#define DOWN_BUTTON		PD0


typedef struct _led {
	uint8_t pos[2];
	volatile uint8_t *port[2];
} led;

led matrix[3][3];
uint8_t board[3][3];

#endif

	volatile uint8_t *A = &PORTA;
	volatile uint8_t *B = &PORTB;
	volatile uint8_t *C = &PORTC;
	volatile uint8_t *D = &PORTD;


int curr_x = 0, curr_y = 0;

/* Functie de initializare a porturilor pentru LED-uri si butoane */
void init_ports()
{
	/* setam directiile pinilor */
	DDRA = 0xFF;
	DDRC = 0xFF;
	DDRB = 0x0F;
	DDRD = 0x00;
	PORTB = 0xFF;

	
	DDRB |= (1 << PB0) | (1 << PB1) | (1 << PB2) | (1 << PB3);
	
	/* activam rezistenta de pull-up pentru butoane */
	PORTD |= (1<<PD0);
	PORTB |= (1<<PB4) | (1<<PB5) | (1<<PB6) | (1<<PB7);
}

/* functie de resetare porturi, la sfarsitul unui joc */
void reset_ports() {

	PORTA = 0;
	PORTC = 0;
	PORTB = 0;
	PORTD = 0;
}


/* functie de aprindere a tuturor ledurilor
*/
void turn_on_all(int color) {
	int i, j;

	for (i = 0; i < 3; i++)
		for (j = 0; j < 3; j++)
			*(matrix[i][j].port[color]) |= (1 << matrix[i][j].pos[color]);
}

void light_led(int i, int j, int color) {

	*(matrix[i][j].port[color]) |= (1 << matrix[i][j].pos[color]);

}
/* functie de stingere a tuturor ledurilor */
void turn_off_all(){
	int i, j;
	for (i = 0; i < 3; i++){
		for (j = 0; j < 3; j++){
			*(matrix[i][j].port[RED]) &= ~(1 << matrix[i][j].pos[RED]);
			*(matrix[i][j].port[GREEN]) &= ~(1 << matrix[i][j].pos[GREEN]);
		}
	}
}


/* functie de stingere a tutuor ledurilor, de o anumita culoare */
void color_out(int color){
	int i, j;
	for (i = 0; i < 3; i++){
		for (j = 0; j < 3; j++){
			*(matrix[i][j].port[color]) &= ~(1 << matrix[i][j].pos[color]);
		}
	}
}




/* functie de apasare a butonului */
int is_pressed(int button) {
	if (bit_is_clear(PINB, button)) {
		_delay_ms(20);
		if (bit_is_clear(PINB, button))
			return 1;
	}
	return 0;
}

/*functie de cautare a urmatorului led liber, la apasarea butonului de selectie */
void get_next_led(int *curr_x, int *curr_y) {
	int i, j;

	for(i = 0; i < 3; i++)
		for (j = 0; j < 3; j++)
			if (board[i][j] == EMPTY) {
				*curr_x = i;
				*curr_y = j;
				return;
			}
}

/* functie de cleanup a matricei */
void cleanup(){
	int i, j;
	for (i = 0; i < 3; i++) {
		for (j = 0; j < 3; j++) {
			board[i][j] = EMPTY;
		}
	}
}


/* functie de gasire a unei noi pozitii in board */
void find_next_pos(int pos, int *curr_x, int *curr_y, int player_move) {
	int i;
	int not_player_move;
	not_player_move = 1 - player_move;
	switch(pos) {
		case RIGHT:
			for (i = 0; i< 3; i++) {
				(*curr_x)++;
				if (*curr_x > 2)
					*curr_x = 0;
				if (board[*curr_x][*curr_y] == EMPTY)
					break;
				else if(board[*curr_x][*curr_y] == not_player_move
						&& (*curr_x) + 1 == 2)
				{
					*curr_x = 2;
				}
					
			}
			break;
			
		case LEFT:
			for (i = 0; i< 3; i++) {
				(*curr_x)--;
				if (*curr_x < 0)
					*curr_x = 2;
				if (board[*curr_x][*curr_y] == EMPTY)
					break;
				else if(board[*curr_x][*curr_y] == not_player_move
						&& (*curr_x) - 1 == 0)
				{
					*curr_x = 0;
				}
			}
			break;
			
		case UP:
			for (i = 0; i< 3; i++) {
				(*curr_y)--;
				if (*curr_y < 0)
					*curr_y = 2;
				if (board[*curr_x][*curr_y] == EMPTY)
					break;
				else if(board[*curr_x][*curr_y] == not_player_move
						&& (*curr_y) - 1 == 0)
				{
					*curr_y = 0;
				}
			}
			break;

		case DOWN:
			for (i = 0; i< 3; i++) {
				(*curr_y)++;
				if (*curr_y > 2)
					*curr_y = 0;
				if (board[*curr_x][*curr_y] == EMPTY)
					break;
				else if(board[*curr_x][*curr_y] == not_player_move
						&& (*curr_y) + 1 == 2)
				{
					*curr_y = 2;
				}
			}
			break;
	}
}

/* functie apelata la fiecare mutare */
void make_move(int player_move, int x){

	*(matrix[curr_x][curr_y].port[player_move]) &= ~(1 << matrix[curr_x][curr_y].pos[player_move]);
	
	switch(x)
	{
		case 0:
			find_next_pos(UP, &curr_x, &curr_y, player_move);
			break;
		case 1:
			find_next_pos(UP, &curr_x, &curr_y, player_move);
			break;
		case 2:
			find_next_pos(RIGHT, &curr_x, &curr_y, player_move);
			break;
		case 3:
			find_next_pos(DOWN, &curr_x, &curr_y, player_move);
			break;
	
	}
		
	*(matrix[curr_x][curr_y].port[player_move]) |= (1 << matrix[curr_x][curr_y].pos[player_move]);
	
	_delay_ms(200);
	

}

/* functie de testare a combinatiei castigatoare */
int test_winner_game(int color, int curr_x, int curr_y) {

	if(board[curr_x][0] == color && 
		board[curr_x][1] ==  color && board[curr_x][2] == color)
	{
		return 1;
	}
	
	else if(board[0][curr_y] == color && 
		board[1][curr_y] ==  color && board[2][curr_y] == color)
	{
		return 1;
	}
	
	else if(board[0][0] == color && 
		board[1][1] ==  color && board[2][2] == color)
	{
		return 1;
	}
	
	else if(board[0][2] == color && 
		board[1][1] ==  color && board[2][0] == color)
	{
		return 1;
	}

	return 0;
}

/* animatie joc castigator */
void winner()
{
	turn_off_all();
	_delay_ms(1000);
	turn_on_all(GREEN);
	_delay_ms(1000);
	color_out(GREEN);
	_delay_ms(1000);
	turn_on_all(RED);
	_delay_ms(1000);
	color_out(RED);


}

/* functie de afisare animatie, pentru remiza */
void draw()
{
	turn_off_all();

	light_led(0,0, GREEN);
	_delay_ms(300);
	light_led(1,0, RED);
	_delay_ms(300);
	light_led(2,0, GREEN);
	_delay_ms(300);
	light_led(2,1, RED);
	_delay_ms(300);
	light_led(1,1, GREEN);
	_delay_ms(300);
	light_led(0,1, RED);
	_delay_ms(300);
	light_led(0,2, GREEN);
	_delay_ms(300);
	light_led(1,2, RED);
	_delay_ms(300);
	light_led(2,2, GREEN);
	_delay_ms(300);
	
	turn_off_all();
	
}



/* animatie sfarsit joc */
void end_the_game()
{
	int i, j;
	int col ;
	for(col =0;col<2;col++)
		for(i=0;i<3;i++)
		{
			for(j=0;j<3;j++)
			{
				*(matrix[i][j].port[col]) |= (1<<matrix[i][j].pos[col]);	
			}
		}

	for(i=0;i<3;i++)
	{
		for(j=0;j<3;j++)
		{
			*(matrix[i][j].port[col]) &= (1<<matrix[i][j].pos[col]);
	
		}
	}
	_delay_ms(500);
	for(i=0;i<3;i++)
	{
		for(j=0;j<3;j++)
		{
			*(matrix[i][j].port[col]) |= (1<<matrix[i][j].pos[col]);
		}
	}
	_delay_ms(500);
	for(i=0;i<3;i++)
	{
		for(j=0;j<3;j++)
		{
			*(matrix[i][j].port[col]) &= (1<<matrix[i][j].pos[col]);
		}
	}
	_delay_ms(500);
}


/* functie de play a jocului */
void play_game() {
	int player_move = RED, moves = 0;

	while (1) {
		if (is_pressed(UP_BUTTON)) { /* daca apasam buton sus */
			make_move(player_move, 0);
			continue;
		}

		if (is_pressed(LEFT_BUTTON)) { /* daca apasam buton stanga */
			make_move(player_move, 3);
			continue;
		}

		if (is_pressed(MOVE_BUTTON)) {  /* daca apasam buton selectie celula */

			*(matrix[curr_x][curr_y].port[player_move]) |= (1 << matrix[curr_x][curr_y].pos[player_move]);
			board[curr_x][curr_y] = player_move;
			/* incrementam numar mutari */
			moves++;
			/* daca avem combinatie castigatoare */
			if (moves >= 5 && test_winner_game(player_move,curr_x, curr_y)) {
				
				winner();				
				return;
			}

			/* remiza */
			if (moves == MAX_MOVES) {
				draw();
				return;
			}

			get_next_led(&curr_x, &curr_y);
			player_move = 1 - player_move;
		}

		if (is_pressed(DOWN_BUTTON)) { /* daca apasam buton jos */
			make_move(player_move, 2);
			continue;
		}
		if (is_pressed(RIGHT_BUTTON)) { /* daca apasam buton dreapta */
			make_move(player_move, 1);
			continue;
		}
		
		*(matrix[curr_x][curr_y].port[player_move]) ^= (1 << matrix[curr_x][curr_y].pos[player_move]);
		_delay_ms(200);
	}
}

/* animatie de inceput X&0 */
void x_o()
{
	turn_off_all();


	light_led(0,0, GREEN);
	_delay_ms(300);
	light_led(1,1, GREEN);
	_delay_ms(300);
	light_led(2,2, GREEN);
	_delay_ms(300);
	light_led(0,2, GREEN);
	_delay_ms(300);
	light_led(2,0, GREEN);
	_delay_ms(300);
	

	turn_off_all();
	_delay_ms(300);
	
	light_led(0,0, RED);
	_delay_ms(300);
	light_led(0,1, RED);
	_delay_ms(300);
	light_led(0,2, RED);
	_delay_ms(300);
	light_led(1,2, RED);
	_delay_ms(300);
	light_led(2,2, RED);
	_delay_ms(300);
	light_led(2,1, RED);
	_delay_ms(300);
	light_led(2,0, RED);
	_delay_ms(300);
	light_led(1,0, RED);
	_delay_ms(300);

	turn_off_all();
}




/* functie de setare matrice de LED-uri pe porturile corespunzatoare */
void init_matrix() {
	matrix[0][0].pos[GREEN] = PA0;
	matrix[0][0].port[GREEN] = &PORTA;
	matrix[0][0].pos[RED] = PA1;
	matrix[0][0].port[RED] = &PORTA;

	matrix[0][1].pos[GREEN] = PA2;
	matrix[0][1].port[GREEN] = &PORTA;
	matrix[0][1].pos[RED] = PA3;
	matrix[0][1].port[RED] = &PORTA;
	
	matrix[0][2].pos[GREEN] = PA4;
	matrix[0][2].port[GREEN] = &PORTA;
	matrix[0][2].pos[RED] = PA5;
	matrix[0][2].port[RED] = &PORTA;

	matrix[1][0].pos[GREEN] = PA6;
	matrix[1][0].port[GREEN] = &PORTA;
	matrix[1][0].pos[RED] = PA7;
	matrix[1][0].port[RED] = &PORTA;

	matrix[1][1].pos[GREEN] = PC4;
	matrix[1][1].port[GREEN] = &PORTC;
	matrix[1][1].pos[RED] = PC3;
	matrix[1][1].port[RED] = &PORTC;
	
	matrix[1][2].pos[GREEN] = PC6;
	matrix[1][2].port[GREEN] = &PORTC;
	matrix[1][2].pos[RED] = PC5;
	matrix[1][2].port[RED] = &PORTC;
	
	matrix[2][0].pos[GREEN] = PC2;
	matrix[2][0].port[GREEN] = &PORTC;
	matrix[2][0].pos[RED] = PC1;
	matrix[2][0].port[RED] = &PORTC;
	
	matrix[2][1].pos[GREEN] = PB0;
	matrix[2][1].port[GREEN] = &PORTB;
	matrix[2][1].pos[RED] = PB1;
	matrix[2][1].port[RED] = &PORTB;
	
	matrix[2][2].pos[GREEN] = PB2;
	matrix[2][2].port[GREEN] = &PORTB;
	matrix[2][2].pos[RED] = PB3;
	matrix[2][2].port[RED] = &PORTB;
}

int main()
{
	
	init_ports();
	init_matrix();
	/* animatie start joc */
	x_o();

	while(1) {

		cleanup();
		play_game();
		while (!is_pressed(PB5))
		;
		reset_ports();
		_delay_ms(300);
	}
	
	return 0;
}
